package Echiquier;

import Echiquier.Modele.Case;
import Echiquier.Modele.ColorJoueur;
import Echiquier.Modele.Piece.*;
import Echiquier.Vue.IHM;

/**
 * La simulation de l'echiquier
 * C'est un partie Controleur.
 */
public class Echiquier {
    public static final int MAXLIG = 8; //nombre max de lignes
    public static final int MAXCOL = 8; //nombre max de colonnes
    //Le Blanche joueur fait le primier deplacement
    private ColorJoueur colorJoueur = ColorJoueur.BLANCHE;
    private Case[][] table;

    public Echiquier(){
        try {
            echiquierInit();
        }catch (Exception e){
            System.out.println(e);
        }

    }

    /**
     * Initialisation de Echiquer, Il cree le table de case.
     * @throws Exception la construction de case
     */
    private void echiquierInit() throws Exception {
        table = new Case[MAXLIG][MAXCOL];

        ColorJoueur colorJoueur = null;
        Piece piece = null;

        for(int lig=0; lig<MAXLIG; lig++) {
            if(lig <= 1){
                colorJoueur = ColorJoueur.NOIR;
            }else {
                colorJoueur = ColorJoueur.BLANCHE;
            }
            for (int col = 0; col < MAXCOL; col++) {
                table[lig][col] = new Case(lig,col);
                //ajouter les pieces dans l'echiquier
                if(lig <= 1 || lig >= 6) {
                    //ajouter les pions
                    if(lig == 1 || lig == 6){
                        piece = new Pion(colorJoueur);
                    }
                    else {
                        //ajouter les tours
                        if(col == 0 || col == 7){
                            piece = new Tour(colorJoueur);
                        }
                        //ajouter les cavaliers
                        if(col == 1 || col == 6){
                            piece = new Cavalier(colorJoueur);
                        }
                        //ajouter les fous
                        if(col == 2 || col == 5){
                            piece = new Fou(colorJoueur);
                        }
                        //ajouter les Dames
                        if(col == 3){
                            piece = new Dame(colorJoueur);
                        }
                        //ajouter les rois
                        if(col == 4){
                            piece = new Roi(colorJoueur);
                        }
                    }

                    piece.setEchiquier(this);
                    table[lig][col].setPiece(piece);
                }
            }
        }
        System.out.println("L'echiquier init");
    }

    /**
     * le deplace de joueur souhaite, il verifier le deplace est valide ou pas
     * @param position position[0] est le deplacement depart souhaitant. position[1] esst le deplacement arrive souhaitant
     * @throws Exception le déplace n'est pas valid.
     */
    public void deplace(Case[] position) throws Exception {
        //recupperer la Case dans l'echiquier
        Case depart = getCase(position[0]);
        Case arrive = getCase(position[1]);

        if(depart.estEuPiece()) {
            if(depart.getPiece().getColorJoueur() != this.colorJoueur)
                throw new Exception(depart.toString() + " n'est pas votre piece");

            if (depart.getPiece().estDpValide(depart,arrive)) {
                //mettre le piece dans le case
                arrive.setPiece(depart.getPiece());
                depart.setPiece(null);
            } else {
                throw new Exception("Il n'est pas possible de déplacer la pièce en " + depart.toString() + " vers " + arrive.toString() + "!");
            }
        }else {
            throw new Exception("Le case" + depart.toString() +  " n'a pas un piece !");
        }

        //Le deplacement reussi, change le joueur deplacant.
        if(this.colorJoueur == ColorJoueur.BLANCHE)
            this.colorJoueur = ColorJoueur.NOIR;
        else if(this.colorJoueur == ColorJoueur.NOIR)
            this.colorJoueur = ColorJoueur.BLANCHE;
    }

    public Case[][] getTable() {
        return table;
    }

    public Case getCase(Case unCase){
        return table[unCase.getLig()][unCase.getCol()];
    }

    public Case getCase(int lig, int col){
        return table[lig][col];
    }

    /**
     * Verifier le deplacement est bloqué.
     * @param depart le case souhaite depart
     * @param arrive le case souhaite arrive
     * @return true il est bloqué
     */
    public boolean estBloqueDiagonales(Case depart, Case arrive){
        //Si l'arrive est en haut à droite
        int nb = depart.getLig() - arrive.getLig();
        if(depart.getLig()-arrive.getLig()>0 && depart.getCol()-arrive.getCol()>0){
            for(int i=1;i<nb; i++){
                if(getCase(depart.getLig()-i,depart.getCol()-i).estEuPiece())
                    return true;
            }
        }

        //Si l'arrive est en haut à gauche
        if(depart.getLig()-arrive.getLig()>0 && depart.getCol()-arrive.getCol()<0){
            for(int i=1;i<nb; i++){
                if(getCase(depart.getLig()-i,depart.getCol()+i).estEuPiece())
                    return true;
            }
        }

        //Si l'arrive est en base à gauche
        if(depart.getLig()-arrive.getLig()<0 && depart.getCol()-arrive.getCol()<0){
            for(int i=1;i<Math.abs(nb); i++){
                if(getCase(depart.getLig()+i,depart.getCol()+i).estEuPiece())
                    return true;
            }
        }

        //Si l'arrive est en base à droite
        if(depart.getLig()-arrive.getLig()<0 && depart.getCol()-arrive.getCol()>0){
            for(int i=1;i<Math.abs(nb); i++){
                if(getCase(depart.getLig()+i,depart.getCol()-i).estEuPiece())
                    return true;
            }
        }

        return false;
    }

    /**
     * Verifier le deplacement est bloqué.
     * @param depart le case souhaite depart
     * @param arrive le case souhaite arrive
     * @return true il est deplacé bloqué
     */
    public boolean estBloqueLigne(Case depart, Case arrive){
        int nbCol = depart.getCol() - arrive.getCol();
        int nbLig = depart.getLig() - arrive.getLig();

        //La deplacement horizontal
        if(depart.getLig() == arrive.getLig()){
            //A droit
            if(nbCol>0){
                for (int i=1;i<nbCol;i++){
                    if(getCase(depart.getLig(), depart.getCol() - i).estEuPiece()){
                        return true;
                    }
                }
            }

            //A gauche
            if(nbCol<0){
                for (int i=1;i<Math.abs(nbCol);i++){
                    if(getCase(depart.getLig(), depart.getCol() + i).estEuPiece()){
                        return true;
                    }
                }
            }
        }

        //La deplacement vertical
        if(depart.getCol() == depart.getCol()){
            //En haut
            if(nbLig<0){
                for (int i=1;i<Math.abs(nbLig); i++){
                    if(getCase(depart.getLig() + i, depart.getCol()).estEuPiece()){
                        return true;
                    }
                }
            }
            //En base
            if(nbLig>0){
                for (int i=1;i<nbLig; i++){
                    if(getCase(depart.getLig() - i, depart.getCol()).estEuPiece()){
                        return true;
                    }
                }
            }
        }

        return false;
    }

    /**
     * Verifier le deplacement est en ligne
     * @param depart le case souhaite depart
     * @param arrive le case souhaite arrive
     * @return true il est en ligne
     */
    public boolean estEnLigne(Case depart,Case arrive){
        return depart.getLig() == arrive.getLig() || depart.getCol() == arrive.getCol();
    }

    /**
     * Verifier le deplacement est diagonales
     * @param depart le case souhaite depart
     * @param arrive le case souhaite arrive
     * @return true il est en diagonaless
     */
    public boolean estEnDiagonales(Case depart, Case arrive){
        return Math.abs(depart.getCol()-arrive.getCol()) == Math.abs(depart.getLig() - arrive.getLig());
    }

    public ColorJoueur getColorJoueur(){
        return this.colorJoueur;
    }

    /**
     * il montre que la joueur exist un roi ou pas
     * Sinon le joeur échec
     * @return true le roi a exist
     */
    public boolean estRoiExist(){
        for(int lig=0;lig<Echiquier.MAXLIG; lig++) {
            for (int col = 0; col < Echiquier.MAXCOL; col++)
                if (table[lig][col].getPiece() instanceof Roi &&
                        table[lig][col].getPiece().getColorJoueur() == this.colorJoueur){
                    return true;
                }
        }
        return false;
    }

    public static void main(String[] argv){
        new IHM(new Echiquier());
    }

}
